@@ -29,6 +29,9 @@ from duplicity import path
 from duplicity import util
 from duplicity.errors import * #@UnusedWildImport
 
+testing_in_progress = False
+
+
 class LocalBackend(duplicity.backend.Backend):
     """Use this backend when saving to local disk
 
@@ -39,11 +42,11 @@ class LocalBackend(duplicity.backend.Backend):
     def __init__(self, parsed_url):
         duplicity.backend.Backend.__init__(self, parsed_url)
         # The URL form "file:MyFile" is not a valid duplicity target.
-        if not parsed_url.path.startswith( '//' ):
-            raise BackendException( "Bad file:// path syntax." )
+        if not parsed_url.path.startswith('//'):
+            raise BackendException("Bad file:// path syntax.")
         self.remote_pathdir = path.Path(parsed_url.path[2:])
 
-    def handle_error(self, e, op, file1=None, file2=None):
+    def handle_error(self, e, op, file1 = None, file2 = None):
         code = log.ErrorCode.backend_error
         if hasattr(e, 'errno'):
             if e.errno == errno.EACCES:
@@ -54,27 +57,39 @@ class LocalBackend(duplicity.backend.Backend):
                 code = log.ErrorCode.backend_no_space
         extra = ' '.join([util.escape(x) for x in [file1, file2] if x])
         extra = ' '.join([op, extra])
-        log.FatalError(str(e), code, extra)
+        if op != 'delete' and op != 'query':
+            log.FatalError(str(e), code, extra)
+        else:
+            log.Warn(str(e), code, extra)
 
-    def put(self, source_path, remote_filename = None, rename = None):
-        """If rename is set, try that first, copying if doesn't work"""
+    def move(self, source_path, remote_filename = None):
+        self.put(source_path, remote_filename, rename_instead = True)
+
+    def put(self, source_path, remote_filename = None, rename_instead = False):
         if not remote_filename:
             remote_filename = source_path.get_filename()
         target_path = self.remote_pathdir.append(remote_filename)
         log.Info("Writing %s" % target_path.name)
-        if rename:
+        """Try renaming first (if allowed to), copying if doesn't work"""
+        if not testing_in_progress:
+            if rename_instead:
+                try:
+                    source_path.rename(target_path)
+                except OSError:
+                    pass
+                except Exception, e:
+                    self.handle_error(e, 'put', source_path.name, target_path.name)
+                else:
+                    return
             try:
-                source_path.rename(target_path)
-            except OSError:
-                pass
+                target_path.writefileobj(source_path.open("rb"))
             except Exception, e:
-                handle_error(e, 'put', source_path.name, target_path.name)
-            else:
-                return
-        try:
-            target_path.writefileobj(source_path.open("rb"))
-        except Exception, e:
-            self.handle_error(e, 'put', source_path.name, target_path.name)
+                self.handle_error(e, 'put', source_path.name, target_path.name)
+
+            """If we get here, renaming failed previously"""
+            if rename_instead:
+                """We need to simulate its behaviour"""
+                source_path.delete()
 
     def get(self, filename, local_path):
         """Get file and put in local_path (Path object)"""
@@ -88,7 +103,7 @@ class LocalBackend(duplicity.backend.Backend):
         """List files in that directory"""
         try:
                 os.makedirs(self.remote_pathdir.base)
-        except:
+        except Exception:
                 pass
         try:
             return self.remote_pathdir.listdir()
@@ -104,5 +119,17 @@ class LocalBackend(duplicity.backend.Backend):
             except Exception, e:
                 self.handle_error(e, 'delete', self.remote_pathdir.append(filename).name)
 
+    def _query_file_info(self, filename):
+        """Query attributes on filename"""
+        try:
+            target_file = self.remote_pathdir.append(filename)
+            if not os.path.exists(target_file.name):
+                return {'size': -1}
+            target_file.setdata()
+            size = target_file.getsize()
+            return {'size': size}
+        except Exception, e:
+            self.handle_error(e, 'query', target_file.name)
+            return {'size': None}
 
 duplicity.backend.register_backend("file", LocalBackend)
