@@ -169,6 +169,33 @@ class DupOption(optparse.Option):
             optparse.Option.take_action(
                 self, action, dest, opt, value, values, parser)
 
+""" 
+Fix:
+    File "/usr/lib/pythonX.X/optparse.py", line XXXX, in print_help
+    file.write(self.format_help().encode(encoding, "replace"))
+    UnicodeDecodeError: 'ascii' codec can't decode byte 0xXX in position XXXX:
+See:
+    http://bugs.python.org/issue2931
+    http://mail.python.org/pipermail/python-dev/2006-May/065458.html
+"""
+class OPHelpFix(optparse.OptionParser):
+    def _get_encoding(self, file):
+        """
+        try to get the encoding or switch to UTF-8
+        which is default encoding in python3 and most recent unixes
+        """
+        encoding = getattr(file, "encoding", "UTF-8")
+        return encoding
+
+    def print_help(self, file=None):
+        """
+        overwrite method with proper utf-8 decoding
+        """
+        if file is None:
+            file = sys.stdout
+        encoding = self._get_encoding(file)
+        file.write(self.format_help().decode('utf-8').encode(encoding, "replace"))
+
 def parse_cmdline_options(arglist):
     """Parse argument list"""
     global select_opts, select_files, full_backup
@@ -211,7 +238,7 @@ def parse_cmdline_options(arglist):
     def add_rename(o, s, v, p):
         globals.rename[os.path.normcase(os.path.normpath(v[0]))] = v[1]
 
-    parser = optparse.OptionParser(option_class=DupOption, usage=usage())
+    parser = OPHelpFix( option_class=DupOption, usage=usage() )
 
     # If this is true, only warn and don't raise fatal error when backup
     # source directory doesn't match previous backup source directory.
@@ -248,6 +275,10 @@ def parse_cmdline_options(arglist):
 
     # secret keyring in which the private encrypt key can be found
     parser.add_option("--encrypt-secret-keyring", type="string", metavar=_("path"))
+    
+    parser.add_option("--encrypt-sign-key", type="string", metavar=_("gpg-key-id"),
+                      dest="", action="callback",
+                      callback=lambda o, s, v, p: ( globals.gpg_profile.recipients.append(v), set_sign_key(v)) )
 
     # TRANSL: Used in usage help to represent a "glob" style pattern for
     # matching one or more files, as described in the documentation.
@@ -288,6 +319,10 @@ def parse_cmdline_options(arglist):
     parser.add_option("--fail-on-volume", type="int",
                       help=optparse.SUPPRESS_HELP)
 
+    # used in testing only - skips upload for a given volume
+    parser.add_option("--skip-volume", type="int",
+                      help=optparse.SUPPRESS_HELP)
+
     # If set, restore only the subdirectory or file specified, not the
     # whole root.
     # TRANSL: Used in usage help to represent a Unix-style path name. Example:
@@ -374,6 +409,9 @@ def parse_cmdline_options(arglist):
     # --num-retries <number>
     parser.add_option("--num-retries", type="int", metavar=_("number"))
 
+    # File owner uid keeps number from tar file. Like same option in GNU tar.
+    parser.add_option("--numeric-owner", action="store_true")
+
     # Whether the old filename format is in effect.
     parser.add_option("--old-filenames", action="callback",
                       dest="old_filenames",
@@ -410,6 +448,12 @@ def parse_cmdline_options(arglist):
     # See <https://bugs.launchpad.net/duplicity/+bug/433970>.
     parser.add_option("--s3-unencrypted-connection", action="store_true")
 
+    # Chunk size used for S3 multipart uploads.The number of parallel uploads to
+    # S3 be given by chunk size / volume size. Use this to maximize the use of
+    # your bandwidth. Defaults to 25MB
+    parser.add_option("--s3-multipart-chunk-size", type="int", action="callback", metavar=_("number"),
+                      callback=lambda o, s, v, p: setattr(p.values, "s3_multipart_chunk_size", v*1024*1024))
+
     # scp command to use
     # TRANSL: noun
     parser.add_option("--scp-command", metavar=_("command"))
@@ -746,6 +790,7 @@ def usage():
   tahoe://%(alias)s/%(directory)s
   webdav://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
   webdavs://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
+  gdocs://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
 
 """ % dict
 
